#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(VramLeft + 1, "SCORE");   
    PrintS(VramLeft + 7, "STAGE");
    PrintByteNumber2(VramLeft + VramRowSize * 1 + 10, CurrentStage + 1);
    PrintS(VramLeft + 13, "TIME");
    {
        ptr<byte> pVram;
        pVram = VramLeft + 19;
        if (RemainCount > 1) {
            static const byte[] Chars = {
                0xb2, 0xac, 0x80, 0x80
            };
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                pVram = Put2S(pVram, Chars);
                pVram = Erase2(pVram);
                PrintC(pVram - 2 + VramRowSize, i + 0x30);
            }
            else {
                do {
                    pVram = Put2S(pVram, Chars);
                    ++pVram;
                    --i;
                } while (i > 0);
            }
        }
    }
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    pVram = PrintNumber5(VramLeft + VramRowSize * 1 + 0, Score);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(VramLeft + VramRowSize * 1 + 14, StageTime);
}

void PrintGameOver()
{
    PrintS(VramLeft + VramRowSize * 8 + 8, "GAME OVER");
    // PresendVram();
}

void PrintTimeUp()
{
    PrintS(VramLeft + VramRowSize * 8 + 9, "TIME UP");
    // PresendVram();
}

void PrintPerfect()
{
    PrintS(VramLeft + VramRowSize * 8 + 9, "PERFECT");
    // PresendVram();
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        static const byte[] TitleBytes = {
            0x80, 0x90, 0x80, 0x20, 
            0x80, 0x91, 0x80, 0x83, 
            0x80, 0x20, 0x80, 0x83, 
            0x90, 0x90, 0xac, 0xab, 
            0x80, 0x80, 0x20, 0x90, 
            0x82, 0x83, 0x82, 0x20, 
            0x80, 0x80, 0x82, 0x20, 
            0xac, 0xab, 0xac, 0x20, 
            0x80, 0x90, 0xab, 0x80, 
            0x80, 0x20, 0x20, 0x83, 
            0x80, 0x20, 0x20, 0x83, 
            0x90, 0x20, 0x20, 0xab, 
            0x80, 0xac, 0x80, 0x20, 
            0x82, 0x20, 0x80, 0x20, 
            0x82, 0x20, 0x80, 0x20, 
            0xac, 0x20, 0x90, 0x90, 
            0x20, 0xa9, 0x80, 0x80, 
            0x20, 0x83, 0x82, 0x83, 
            0x20, 0x83, 0x82, 0x83, 
            0x90, 0x20, 0x90, 0x90, 
            0xaa, 0x90, 0x80, 0x90, 
            0x82, 0x20, 0x80, 0x20, 
            0x82, 0x20, 0x80, 0x20, 
            0x20, 0x20, 0x90, 0x20, 
        };
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = VramLeft + VramRowSize * 3 + 0;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p; ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
        PrintS(VramLeft + VramRowSize * 7 + 18, "MINI");
    }

    PrintS(VramLeft + VramRowSize * 12 + 5, "PUSH SPACE KEY");
    PrintS(VramLeft + VramRowSize * 13 + 5, "OR 'Z' KEY");
    PrintS(VramLeft + VramRowSize * 15 + 12, "INUFUTO 2024");

    // PresendVram();
}